//******************************************************************************
//Name          : RandomSimulation.cs
//Description   : Implementation to experience the beauty and the pitfalls of Random
//                Simulation
//Author        : Dr. Oliver Kluge, 90411 Nuernberg, Germany
//Copyright(C)  : Oliver Kluge 2014 
//------------------------------------------------------------------------------
//******************************************************************************

using System;


public class RandomSimulation
{

    public double MonteCarloPi(long count)
    {
        MyRandom.Random noise = new MyRandom.Random(0.71036, 0.23906);
        double r1 = 0;
        double r2 = 0;
        long incircle = 0;

        for (long k = 0; k < count; k++)
        {
            // create 2 random numbers [0, 1) => [-1, 1)
            r1 = 2*noise.Uniform1() - 1;
            r2 = 2*noise.Uniform2() - 1;

            // check if r1 and r2 are within the unit circle
            if (Math.Sqrt(r1*r1 + r2*r2) <= 1.0)
            {
                incircle++;
            }
        }

        return (4.0*((double) incircle)/((double) count));
    }

    public void CheckCycle(long count)
    {
        MyRandom.Random noise = new MyRandom.Random(0.71036, 0.23906);
        bool foundcycle = false;
        double startvalue = noise.Uniform1();
        double value = 0;

        for (long k = 0; k < count; k++)
        {
            value = noise.Uniform1();

            if (Math.Abs(value - startvalue) < 1E-12)
            {
                Console.WriteLine("started with {0}, now created {1} at k={2}", startvalue, value, k);
                foundcycle = true;
                break;
            }
        }

        if (!foundcycle)
        {
            Console.WriteLine("CheckCycle(): no cycle found!");
        }
    }
}

class MainApp
{
    [STAThread]
    static void Main()
    {
        //string userinput = "";
        //int select = -1;
        RandomSimulation mySimulation = new RandomSimulation();

        Console.WriteLine("Calculate PI using MonteCarlo-algorithm");
        Console.WriteLine();

        Console.WriteLine("n=10         -> PI={0}", mySimulation.MonteCarloPi(10));
        Console.WriteLine("n=100        -> PI={0}", mySimulation.MonteCarloPi(100));
        Console.WriteLine("n=1000       -> PI={0}", mySimulation.MonteCarloPi(1000));
        Console.WriteLine("n=10000      -> PI={0}", mySimulation.MonteCarloPi(10000));
        Console.WriteLine("n=100000     -> PI={0}", mySimulation.MonteCarloPi(100000));
        Console.WriteLine("n=1000000    -> PI={0}", mySimulation.MonteCarloPi(1000000));
        Console.WriteLine("n=10000000   -> PI={0}", mySimulation.MonteCarloPi(10000000));
        Console.WriteLine("n=100000000  -> PI={0}", mySimulation.MonteCarloPi(100000000));
        Console.WriteLine("n=1000000000 -> PI={0}", mySimulation.MonteCarloPi(1000000000));

        //mySimulation.CheckCycle(1000000000);
        
        Console.WriteLine();
        Console.WriteLine("...press <Enter>\n");
        Console.ReadLine();
    }
}
